//[navigation-core](../../index.md)/[com.chrynan.navigation](index.md)

# Package-level declarations

## Types

| Name | Summary |
|---|---|
| [DestinationAndContext](-destination-and-context/index.md) | [common]<br>@Serializable<br>class [DestinationAndContext](-destination-and-context/index.md)&lt;[Destination](-destination-and-context/index.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [Context](-destination-and-context/index.md) : [NavigationContext](-navigation-context/index.md)&lt;[Destination](-destination-and-context/index.md)&gt;&gt;(val destination: [Destination](-destination-and-context/index.md), val context: [Context](-destination-and-context/index.md))<br>A wrapper class around a [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696) and an associated [NavigationContext](-navigation-context/index.md). |
| [ExperimentalNavigationApi](-experimental-navigation-api/index.md) | [common]<br>@[Target](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin.annotation/-target/index.html)(allowedTargets = [[AnnotationTarget.CLASS](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin.annotation/-annotation-target/-c-l-a-s-s/index.html), [AnnotationTarget.FUNCTION](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin.annotation/-annotation-target/-f-u-n-c-t-i-o-n/index.html), [AnnotationTarget.FIELD](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin.annotation/-annotation-target/-f-i-e-l-d/index.html), [AnnotationTarget.PROPERTY](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin.annotation/-annotation-target/-p-r-o-p-e-r-t-y/index.html)])<br>annotation class [ExperimentalNavigationApi](-experimental-navigation-api/index.md)<br>Indicates that the component annotated with this annotation is experimental and could change or be removed in the future. |
| [NavigationContext](-navigation-context/index.md) | [common]<br>interface [NavigationContext](-navigation-context/index.md)&lt;[Destination](-navigation-context/index.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696)&gt;<br>Represents a navigation context, or a container of a back stack of [Destination](-navigation-context/index.md)s. Navigation can take place within a [NavigationContext](-navigation-context/index.md) typically by changing [Destination](-navigation-context/index.md) values. But an application may have multiple [NavigationContext](-navigation-context/index.md)s that can be changed and navigated through. This allows for more complex navigation paradigms, such as retaining multiple back stacks for a bottom navigation UI component. |
| [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696) | [common]<br>typealias [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696) = [Any](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-any/index.html)<br>Represents an intent to move to a different screen in the application. This is similar to an Intent from the MVI design pattern, but for navigation purposes.<br>For example:<br>```kotlin sealed class HomeScreenDestination {<br>    object Feed : HomeScreenDestination() } ``` |
| [NavigationEvent](-navigation-event/index.md) | [common]<br>@Serializable(with = [NavigationEventSerializer::class](../../../navigation-core/com.chrynan.navigation/-navigation-event-serializer/index.md))<br>sealed class [NavigationEvent](-navigation-event/index.md)&lt;[D](-navigation-event/index.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [C](-navigation-event/index.md) : [NavigationContext](-navigation-context/index.md)&lt;[D](-navigation-event/index.md)&gt;&gt;<br>Represents a navigation event that is sent to a [Navigator](-navigator/index.md) to coordinate the navigation between UI components, such as &quot;screens&quot; within an application. A [NavigationEvent](-navigation-event/index.md) can be a [NavigationEvent.Forward.Destination](-navigation-event/-forward/-destination/index.md) representing a change in a [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696) in the current context, a [NavigationEvent.Forward.Context](-navigation-event/-forward/-context/index.md) representing a change in [NavigationContext](-navigation-context/index.md), or a [NavigationEvent.Backward](-navigation-event/-backward/index.md) representing a back tracking of a previous [NavigationEvent](-navigation-event/index.md). |
| [NavigationState](-navigation-state/index.md) | [common]<br>@Serializable(with = [NavigationStateSerializer::class](../../../navigation-core/com.chrynan.navigation/-navigation-state-serializer/index.md))<br>interface [NavigationState](-navigation-state/index.md)&lt;[T](-navigation-state/index.md)&gt;<br>A generic wrapper around the state of a navigation component. This provides a way to access the retained [initial](-navigation-state/initial.md) state value, the [current](-navigation-state/current.md) state value, and [changes](-navigation-state/changes.md) to the state value. |
| [NavigationStateStore](-navigation-state-store/index.md) | [common]<br>@Serializable(with = [NavigationStateStoreSerializer::class](../../../navigation-core/com.chrynan.navigation/-navigation-state-store-serializer/index.md))<br>interface [NavigationStateStore](-navigation-state-store/index.md)&lt;[Destination](-navigation-state-store/index.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [Context](-navigation-state-store/index.md) : [NavigationContext](-navigation-context/index.md)&lt;[Destination](-navigation-state-store/index.md)&gt;&gt;<br>Represents a store of navigation state information that is useful for a [Navigator](-navigator/index.md). |
| [NavigationStrategy](-navigation-strategy/index.md) | [common]<br>interface [NavigationStrategy](-navigation-strategy/index.md)<br>A component that encapsulates the various navigation policies for a [Navigator](-navigator/index.md). |
| [NavigationViewModel](-navigation-view-model/index.md) | [common]<br>expect abstract class [NavigationViewModel](-navigation-view-model/index.md)<br>Represents a &quot;ViewModel&quot; component used for navigation purposes. This component should not be used outside this navigation library.<br>[android, js, jvm, native]<br>[android]<br>actual abstract class [NavigationViewModel](-navigation-view-model/index.md) : [ViewModel](https://developer.android.com/reference/kotlin/androidx/lifecycle/ViewModel.html)<br>[js, jvm, native]<br>actual abstract class [NavigationViewModel](-navigation-view-model/index.md) |
| [Navigator](-navigator/index.md) | [common]<br>@[ExperimentalNavigationApi](-experimental-navigation-api/index.md)<br>@Serializable(with = [NavigatorSerializer::class](../../../navigation-core/com.chrynan.navigation/-navigator-serializer/index.md))<br>interface [Navigator](-navigator/index.md)&lt;[Destination](-navigator/index.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [Context](-navigator/index.md) : [NavigationContext](-navigation-context/index.md)&lt;[Destination](-navigator/index.md)&gt;&gt;<br>A [Navigator](-navigator/index.md) is responsible for coordinating the navigation between the different UI component groupings in an application. It is a stateful component that reacts to [NavigationEvent](-navigation-event/index.md)s that are emitted via calls to the navigation functions ([push](push.md), [popDestination](pop-destination.md), and [push](push.md)) and updates its stored state values which can be accessed via its state [store](-navigator/store.md). It is up to the user of a [Navigator](-navigator/index.md) to subscribe to the state changes of this component and update the associated UI accordingly. |
| [SingleNavigationContext](-single-navigation-context/index.md) | [common]<br>@Serializable<br>class [SingleNavigationContext](-single-navigation-context/index.md)&lt;[Destination](-single-navigation-context/index.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696)&gt; : [NavigationContext](-navigation-context/index.md)&lt;[Destination](-single-navigation-context/index.md)&gt; <br>An implementation of the [NavigationContext](-navigation-context/index.md) interface that doesn't have multiple contexts. Typically, a [NavigationContext](-navigation-context/index.md) will either be a sealed class or an enum representing the different contexts for navigation. This is common, for instance, for a UI with a bottom navigation bar, where each navigation item in that bottom navigation bar component would be a different context. Each context would retain its own stack of destinations in the [Navigator](-navigator/index.md). However, sometimes it may be preferable to have only a single context for navigation, and in this case, this class can be used. |

## Functions

| Name | Summary |
|---|---|
| [changeContext](change-context.md) | [common]<br>@[ExperimentalNavigationApi](-experimental-navigation-api/index.md)<br>fun &lt;[Destination](change-context.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [Context](change-context.md) : [NavigationContext](-navigation-context/index.md)&lt;[Destination](change-context.md)&gt;&gt; [Navigator](-navigator/index.md)&lt;[Destination](change-context.md), [Context](change-context.md)&gt;.[~~changeContext~~](change-context.md)(context: [Context](change-context.md)): [Boolean](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-boolean/index.html)<br>Convenience function for the [Navigator.push](push.md) function. |
| [goTo](go-to.md) | [common]<br>@[ExperimentalNavigationApi](-experimental-navigation-api/index.md)<br>fun &lt;[Destination](go-to.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [Context](go-to.md) : [NavigationContext](-navigation-context/index.md)&lt;[Destination](go-to.md)&gt;&gt; [Navigator](-navigator/index.md)&lt;[Destination](go-to.md), [Context](go-to.md)&gt;.[~~goTo~~](go-to.md)(destination: [Destination](go-to.md)): [Boolean](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-boolean/index.html)<br>Convenience function for the [Navigator.push](push.md) function. |
| [Navigator](-navigator.md) | [common]<br>@[ExperimentalNavigationApi](-experimental-navigation-api/index.md)<br>fun &lt;[Destination](-navigator.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [Context](-navigator.md) : [NavigationContext](-navigation-context/index.md)&lt;[Destination](-navigator.md)&gt;&gt; [Navigator](-navigator.md)(initialContext: [Context](-navigator.md), duplicateDestinationStrategy: [NavigationStrategy.DuplicateDestination](-navigation-strategy/-duplicate-destination/index.md) = NavigationStrategy.DuplicateDestination.ALLOW_DUPLICATES, destinationRetentionStrategy: [NavigationStrategy.DestinationRetention](-navigation-strategy/-destination-retention/index.md) = NavigationStrategy.DestinationRetention.RETAIN): [Navigator](-navigator/index.md)&lt;[Destination](-navigator.md), [Context](-navigator.md)&gt;<br>Creates a [Navigator](-navigator/index.md) instance with the provided values.<br>[common]<br>@[ExperimentalNavigationApi](-experimental-navigation-api/index.md)<br>fun &lt;[Destination](-navigator.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696)&gt; [Navigator](-navigator.md)(initialDestination: [Destination](-navigator.md), duplicateDestinationStrategy: [NavigationStrategy.DuplicateDestination](-navigation-strategy/-duplicate-destination/index.md) = NavigationStrategy.DuplicateDestination.ALLOW_DUPLICATES, destinationRetentionStrategy: [NavigationStrategy.DestinationRetention](-navigation-strategy/-destination-retention/index.md) = NavigationStrategy.DestinationRetention.RETAIN): [Navigator](-navigator/index.md)&lt;[Destination](-navigator.md), [SingleNavigationContext](-single-navigation-context/index.md)&lt;[Destination](-navigator.md)&gt;&gt;<br>Creates a [Navigator](-navigator/index.md) instance with the provided values using the [SingleNavigationContext](-single-navigation-context/index.md). |
| [popContext](pop-context.md) | [common]<br>@[ExperimentalNavigationApi](-experimental-navigation-api/index.md)<br>fun &lt;[Destination](pop-context.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [Context](pop-context.md) : [NavigationContext](-navigation-context/index.md)&lt;[Destination](pop-context.md)&gt;&gt; [Navigator](-navigator/index.md)&lt;[Destination](pop-context.md), [Context](pop-context.md)&gt;.[popContext](pop-context.md)(): [Boolean](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-boolean/index.html)<br>Performs a back navigation operation, if possible, by moving to the previous [NavigationContext](-navigation-context/index.md). If this [Navigator](-navigator/index.md) cannot navigate to a previous [NavigationContext](-navigation-context/index.md), then this function will do nothing and return `false`. |
| [popDestination](pop-destination.md) | [common]<br>@[ExperimentalNavigationApi](-experimental-navigation-api/index.md)<br>fun &lt;[Destination](pop-destination.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [Context](pop-destination.md) : [NavigationContext](-navigation-context/index.md)&lt;[Destination](pop-destination.md)&gt;&gt; [Navigator](-navigator/index.md)&lt;[Destination](pop-destination.md), [Context](pop-destination.md)&gt;.[popDestination](pop-destination.md)(): [Boolean](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-boolean/index.html)<br>Performs a back navigation operation, if possible, by removing the top [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696) within the current [NavigationContext](-navigation-context/index.md). If this [Navigator](-navigator/index.md) cannot navigate to a previous [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), then this function will do nothing and return `false`. |
| [push](push.md) | [common]<br>@[ExperimentalNavigationApi](-experimental-navigation-api/index.md)<br>fun &lt;[Destination](push.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [Context](push.md) : [NavigationContext](-navigation-context/index.md)&lt;[Destination](push.md)&gt;&gt; [Navigator](-navigator/index.md)&lt;[Destination](push.md), [Context](push.md)&gt;.[push](push.md)(context: [Context](push.md)): [Boolean](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-boolean/index.html)<br>Changes the current [Context](push.md) to the provided [context](push.md) value. The displayed [Destination](push.md) will be the top destination value in the stack associated with the provided [context](push.md), or the provided context's [NavigationContext.initialDestination](-navigation-context/initial-destination.md) if there is currently no existing stack for the provided [context](push.md).<br>[common]<br>@[ExperimentalNavigationApi](-experimental-navigation-api/index.md)<br>fun &lt;[Destination](push.md) : [NavigationDestination](index.md#1223765350%2FClasslikes%2F-215881696), [Context](push.md) : [NavigationContext](-navigation-context/index.md)&lt;[Destination](push.md)&gt;&gt; [Navigator](-navigator/index.md)&lt;[Destination](push.md), [Context](push.md)&gt;.[push](push.md)(destination: [Destination](push.md)): [Boolean](https://kotlinlang.org/api/latest/jvm/stdlib/kotlin/-boolean/index.html)<br>Navigates to the provided [destination](push.md) in the current [NavigationContext](-navigation-context/index.md). Depending on the provided [NavigationStrategy.DuplicateDestination](-navigation-strategy/-duplicate-destination/index.md) when creating this [Navigator](-navigator/index.md), and the current [Context](push.md) stack, this will either clear the current [Context](push.md) stack to the last value that equals the provided [destination](push.md), or add the provided [destination](push.md) to the top of the current [Context](push.md) stack. |
